import requests
import zipfile
from lr_etl.Config import *
import io


class ExtractDataSource:

    def __init__(self, link) -> None:
        self.link = link

    def extract(self) -> None:
        """Realiza o download do arquivo zip contendo as listas no formato CSV do servidor FTP da Fira"""
        r = requests.get(self.link, headers=HEADER, verify=False)
        z = zipfile.ZipFile(io.BytesIO(r.content))
        z.extractall(STORAGE_FOLDER)



from sqlalchemy import orm
from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.ext.declarative import declarative_base
import sqlalchemy as sa
from sqlalchemy_utils import database_exists, create_database
from datetime import datetime as dt



class DbSqlA():
    base        = declarative_base()
    def __init__(self,ConnectionString):
        self.engine                 = sa.create_engine(ConnectionString, pool_size=40, max_overflow=40)
        self.session                = scoped_session(sessionmaker(autocommit=False,autoflush=False,bind=self.engine))
        self.base.query             = self.session.query_property()
        self.orm_session            = orm.scoped_session(orm.sessionmaker())(bind=self.engine)
        self.base.metadata.bind     = self.engine    

    def create_all(self):
        if not database_exists(self.engine.url):
            create_database(self.engine.url)
        self.base.metadata.create_all(self.engine)






import logging
from dataclasses import asdict
from lr_etl.Models.entities.log_model import LogModel
from lr_etl.Config import *
from functools import wraps
from lr_etl.Libs.date import DateUteis
import requests
import json, os
from timeit import default_timer as timer
from datetime import timedelta
import psutil


class Log:

    Cliente: str = None
    BASE_API: str = None
  
    log_format = 'Data/Hora: %(asctime)s | level: %(levelname)s | mensagem: %(message)s'
    logging.basicConfig(filename='log_atualizacao_listas.log', level=logging.INFO, format=log_format)
    logger = logging.getLogger('root')

    global_exceptions = []
    has_exceptions: bool = False

    def setBaseUrl(cls, url: str):
        cls.BASE_API = url

    def setClienteName(cls, name):
        cls.Cliente = name

    @classmethod
    def info(cls, message):
        cls.logger.info(message)
        cls.global_exceptions.append(message)
        
    @classmethod
    def warning(cls, message):
        cls.logger.warning(message)
        cls.global_exceptions.append(message)
        cls.has_exceptions = True

    @classmethod
    def post_log(cls, model: LogModel):
        try:
            cls.info("enviando log...")
            d = asdict(model)
            response = requests.post(cls.BASE_API + "log-robos", headers={"Content-Type": "application/json"}, data=json.dumps(d))
            if response.status_code == 200:
                cls.info("log enviado...")
            else:
                cls.info("falha ao enviar o log, message: {}".format(response.content))
        except Exception as err:
            cls.warning("erro ao enviar log: {}".format(str(err)))

def LogWrapping(*args, **kwargs):
    def wrapper(func):
        @wraps(func)
        def wrapped(*farg, **fkw):
            try:
                start = timer()

                retorno = None
                trace = None

                main = func.__name__
                Log.info("{} started".format(main))
                retorno = func(*farg)
                Log.info("{} finished".format(main))

                status = "SUCESSO"
               
            except Exception as err:
                Log.warning(str(err))
                status = "FALHA"
                trace = str(err)
            finally:
                log = LogModel(
                    NomeProcesso="LR",
                    Cliente=Log.Cliente,
                    Ambiente=ENV,  
                    Funcao=func.__name__,
                    Status=status,
                    MessageError=trace,
                    DataExecucao=DateUteis.now(fmt="%Y-%m-%d %H:%M"),
                    VM=os.getenv('COMPUTERNAME'),
                    Perfil=os.getlogin(),
                    Duracao=str(timedelta(seconds=timer()-start))[:-3],
                    ConsumoCPU=psutil.cpu_percent(4),
                    ConsumoMemoria=psutil.virtual_memory()[2],
                    Diretorio=os.getcwd(),
                    Detalhes=retorno if main == "insert" and status == "SUCESSO" else None
                )
                Log.post_log(log)
            return retorno
               
        return wrapped
    return wrapper







from datetime import datetime as dt, timedelta as td
from datetime import datetime
from xmlrpc.client import DateTime


# COLOCANDO AQUI FICA GENERICO E VOCE PODE PASSAR PARA OUTROS PROJETOS

class DateUteis():
    def now(fmt=None):
        v =  dt.now()
        return v if not fmt else v.strftime(fmt)

    def getDateRange(startAt=None,days=0,fmt=None):
        today = DateUteis.today() #
        dates = []

        if days >= 0:
            dates = [DateUteis.dateAddDays(today,x,fmt=fmt) for x in range(0, days)]
        else:
            dates = [DateUteis.dateAddDays(today,x*-1,fmt=fmt) for x in range(0, abs(days))]
        return dates 

    def today(fmt=None,addDays=0):
        v =  dt.now().date() 
        v =  v if not addDays else DateUteis.dateAddDays(v,addDays)
        return v if not fmt else v.strftime(fmt)

    def yesterday(fmt=None):
        v = dt.now() - td(1)
        return v if not fmt else v.strftime(fmt)

    def dateAddDays(date:datetime,days:int,fmt=None):
        v = date + td(days) if days > 0 else date - td(abs(days))
        return v if not fmt else v.strftime(fmt)

    def lastWorkingDate(ref:datetime=None,fmt=None): #IGNORE MONDAY AND SUNDAY
        self = DateUteis
        y = self.yesterday() if not ref else self.dateAddDays(ref,-1)
        if y.weekday() in [5,6]:
            return self.lastWorkingDate(self.dateAddDays(y,-1),fmt)
        return y if not fmt else y.strftime(fmt)
    

# d = DateUteis()
# a = d.lastWorkingDate(fmt="%Y-%m-%d")
# a=1



from dataclasses import dataclass

@dataclass
class DBConfig:
    db_engine: str = None
    USER: str = None
    PASSWORD: str = None
    SERVER: str = None
    DB_NAME: str = None
    DRIVER: str = None


import sqlalchemy as sa
from lr_etl.Models.db import sql_server

class auxDB():
    id = sa.Column(sa.Integer,primary_key=True,autoincrement=True)

    def save(self):
        try:
            if not self.id:
                sql_server.session.add(self)
            sql_server.session.commit()
        except:
            sql_server.session.rollback()
            raise
        
    
    def delete(self):
        sql_server.session.rollback()
        sql_server.session.delete(self)
        sql_server.session.commit()




from lr_etl.Config import *
from lr_etl.Models.db.db_handler import Database, DBConfig
from typing import List
from pandas import DataFrame
from lr_etl.use_cases import *
from lr_etl.Libs.file import FileUteis
from lr_etl.Libs.log import LogWrapping, Log
from lr_etl.Models.adapter import DataAdapter
from alive_progress import alive_bar
from lr_etl.Config import *


class LR:

    """
    Handler to process extract, transform and data load.

    LR -> v1
    LA -> v2
    """

    @LogWrapping()
    def handle(config):
        """
         :Basic flow
        -----------
        1. download csv files from a external link which will provided.
        2. read and transform all data source files to dataframes.
        3. create the database connection.
        4. for each dataframe: 
            4.1 adapter the attributes to works with any process type (LR or LA)
            4.2 load the dataframe to database.
        5. clear storage folder.

        Parameters
        ----------
        config: GeneralConfig
            all settings necessary to process works.

        """

        ### Set configs on log handler
        Log.BASE_API = config.apiBaseUrl
        Log.Cliente = config.clienteName

        ### extract data
        # extractor = ExtractDataSource(config.linkDownload)
        # extractor.extract()

        ### parse data: convert and format into dataframes
        parser = ParseData()
        datasets: List[DataFrame] = parser.parse()

        ### create the database connection
        db = Database(config.dbConfig)
        conn = db.connect()

        ### repository handler which will realize databse operations
        repository = Repository(conn, config.tablename)

        with alive_bar(len(datasets)) as bar:
            for df in datasets:
                info = DataAdapter(conn, df, config.dataVersion)
                repository.load(df, info)

                # if config.dataVersion == "v2":
                #     repository.updateManagementTables()
                #     ExecuteRebuild.handle(config.weekday)

                bar()

        FileUteis.clearFolder(STORAGE_FOLDER)
            


from dataclasses import dataclass
from lr_etl.Models.entities.db_config import DBConfig

@dataclass
class GeneralConfig:
    apiBaseUrl: str
    clienteName: str
    linkDownload: str
    tablename: str
    dataVersion: str
    weekday: int
    dbConfig: DBConfig


import pandas as pd


class DataAdapter:
    ids: int
    name: str

    def __init__(self, conn, df, dataVersion) -> None:
        if dataVersion == "v2":
            self.ids = df["IdSublista"].iloc[0]
            sub = pd.read_sql("ListaAtencaoSublista", conn.engine)
            subf = sub[sub["Id"] == self.ids]
            self.name = subf["Nome"].iloc[0]
        else:
            self.ids = df["ID_LISTA"].iloc[0]
            self.name = df["LISTA"].iloc[0]






from lr_etl.Libs.log import Log, LogWrapping
from lr_etl.Models.adapter import DataAdapter
from lr_etl.Config import *
import numpy
from pandas import DataFrame


class Repository:

    def __init__(self, conn, tablename) -> None:
        self.tablename = tablename
        self.conn = conn

    @LogWrapping()
    def delete(self, ids):
        query = "DELETE FROM " + self.tablename + \
                "WHERE" + TABLEIDCOL[self.tablename] + " = " + str(ids)

        self.conn.session.execute(query)
        self.conn.session.commit()

    @LogWrapping()
    def insert(self, df: DataFrame, info: DataAdapter):
        df.to_sql(
            name=self.tablename,
            con=self.conn.engine,
            if_exists='replace',
            index=False,
            method="multi",
            chunksize=100
        )
        return "id: {} name: {}".format(info.ids, info.name)

    def load(self, df: DataFrame, info: DataAdapter):
        # self.delete(info.ids)

        if df.shape[0] > 200000:
            groups = numpy.array_split(df, 50)
            for group in groups: 
                self.insert(group, info)
        else:
            self.insert(df, info)

        



from dataclasses import dataclass, asdict
from datetime import datetime

@dataclass
class LogModel:
    NomeProcesso: str
    Cliente: str
    Ambiente: str
    VM: str
    Perfil: str
    DataExecucao: datetime
    Funcao: str
    Duracao: str
    Status: str
    MessageError: str
    ConsumoCPU: str
    ConsumoMemoria: str
    Diretorio: str
    Detalhes: str


from os import listdir
import os
from typing import List
from pandas import DataFrame
from lr_etl.Libs.log import Log, LogWrapping
from lr_etl.Libs.date import DateUteis
from lr_etl.Models.db import sql_server
from lr_etl.Config import *
import pandas as pd
import requests
import zipfile
import io
import numpy
import os 
from lr_etl.Config import *

def atualizar_gestao_de_listas():
    storage = os.listdir(STORAGE_FOLDER)
    if "tbl_sublista.csv" in storage:
        atualizar_sublista()

def carregar_sublista(conn):
    df = pd.read_sql("ListaAtencaoSublista", conn.engine)
    return df

@LogWrapping()
def download_do_arquivo() -> None:
    """Realiza o download do arquivo zip contendo as listas no formato CSV do servidor FTP da Fira"""
    r = requests.get(DOWNLOAD_LISTA, headers=HEADER, verify=False)
    z = zipfile.ZipFile(io.BytesIO(r.content))
    z.extractall(STORAGE_FOLDER)

@LogWrapping()
def converter_arquivos_para_dataframes() -> List[DataFrame]:
    """Converte os arquivos csv das listas em uma lista de dataframes"""
    listasStorage = listdir(STORAGE_FOLDER)
    listasStorage = [item for item in listasStorage if '.csv' in item and item not in ['tbl_periodicidade.csv', 'tbl_sublista.csv', 'lista_atencao.csv', 'tbl_fontes.csv']]
    listas = [pd.read_csv(os.path.join(STORAGE_FOLDER, item), encoding="utf8") for item in listasStorage]
    for i, df in enumerate(listas):
        if 'Unnamed: 0' in df.columns:
            listas[i] = df.drop(columns=df.columns[0])
    return listas

@LogWrapping()
def atualizar_lista(nome, ids, df) -> None:
    """Realizar o delete e o insert do df atualizado nas listas não diárias."""
    Log.info('Deletando registros existentes...')

    sql_server.session.execute(f'delete from {tblListaAtencao} where IdSublista = {int(ids)}')
    sql_server.session.commit()

    Log.info('Inserindo dados atualizados...')
    if df.shape[0] > 200000:
        groups = numpy.array_split(df, 50)
        for group in groups:
            group.to_sql(tblListaAtencao, sql_server.engine, if_exists='append', index=False, method="multi", chunksize=100)
    else:
        df.to_sql(tblListaAtencao, sql_server.engine, if_exists='append', index=False, method="multi", chunksize=100)

@LogWrapping()
def atualizar_sublista() -> None:
    """Realizar a migração dos dados da tabela ListaAtencaoSublista"""
    df = pd.read_csv(os.path.join(STORAGE_FOLDER, "tbl_sublista.csv"), encoding='utf8')
    sub = pd.read_sql("ListaAtencaoSublista", sql_server.engine)
    
    ids = sub["Id"].tolist()
    for i, row in df.iterrows():
        if int(row["Id"]) in list(map(int, ids)):
            dtatt = row["DataAtualizacaoDados"]
            dtcol = row["DataColetaDados"]
            periodo = row["IdPeriodicidade"]
            fonte = row["IdFonte"]
            id_s = row['Id']

            sql_server.session.execute(f"UPDATE ListaAtencaoSublista SET DataAtualizacaoDados = '{dtatt}' WHERE Id = {id_s}")
            sql_server.session.commit()

            sql_server.session.execute(f"UPDATE ListaAtencaoSublista SET DataColetaDados = '{dtcol}' WHERE Id = {id_s}")
            sql_server.session.commit()

            sql_server.session.execute(f"UPDATE ListaAtencaoSublista SET IdPeriodicidade = {periodo} WHERE Id = {id_s}")
            sql_server.session.commit()

            sql_server.session.execute(f"UPDATE ListaAtencaoSublista SET IdFonte = {fonte} WHERE Id = {id_s}")
            sql_server.session.commit()
        else:
            newdf = df[df["Id"] == int(row[0])]
            del newdf["Id"]
            newdf.to_sql("ListaAtencaoSublista", sql_server.engine, if_exists="append", index=False)

@LogWrapping()
def executar_rebuild():
    sql_server.session.execute("SET NOCOUNT ON; EXEC PR_INDEX :p1", {"p1": tblListaAtencao})
    sql_server.session.commit()


import os
import glob
import shutil
import time
from lr_etl.Libs.log import Log

class FileUteis():

    def createFolder(folder,replace=False):
        if not os.path.isdir(folder):
            os.mkdir(folder)
        if replace:
            os.remove(folder)
            os.mkdir(folder)

    def moveFiles(pattern, place):
        files = glob.glob(pattern)
        for f in files:
            fnm = os.path.basename(f)
            # shutil.copy(f, DIR +  fr'\{place}.xls')
            shutil.copy(f,f"{place}/{fnm}")
            os.remove(f)

    def clearFolder(folder):
        files = glob.glob(folder + "\*.*")
        for f in files:
            try:
                os.remove(f)
            except OSError as e:
                Log.warning(f"erro ao excluir arquivos do storage: {str(e)}")

    def waitFileFromChrome(pattern,dirr=None,timeout=0):
        start = time.time()
        dirr =  dirr if dirr else fr'c:\Users\{os.getlogin()}\downloads'
        
        while not glob.glob(f"{dirr}\{pattern}"):
            if timeout and time.time() - start > timeout: raise("Timeout downloading the file")
        
        time.sleep(0.5)

        while glob.glob(dirr + "\*.crdownload"):
            time.sleep(0.5)
            if timeout and time.time() - start > timeout: raise("Timeout downloading the file")
        
        list_of_files = glob.glob(f"{dirr}\{pattern}") # * means all if need specific format then *.csv
        return max(list_of_files, key=os.path.getctime)

    def renameFile(fl,newName):
        path = os.path.dirname(os.path.abspath(fl))
        os.rename(fl,f"{path}\{newName}")
        return f"{path}\{newName}"

    def getFileName(addrs):
        return os.path.basename(addrs)
        



# d = DateUteis()
# a = d.lastWorkingDate(fmt="%Y-%m-%d")
# a=1



import os 

# log config
HEADER = {'User-Agent': 'Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.36'}

BASE_PATH = os.getcwd()
ENV = "PRD"

# paths
STORAGE_FOLDER = os.path.join(BASE_PATH, "storage")
BCP_LOG_PATH = os.path.join(BASE_PATH, "imports")
IMPORT_BAT = os.path.join(BASE_PATH, "import.bat")


TABLEIDCOL = {"ST_LISTA_RESTRITIVA_FIRA": "ID_LISTA", "ListaAtencao": "IdSublista"}





import urllib
from lr_etl.Models.entities.db_config import DBConfig
from lr_etl.Libs.log import LogWrapping
from sqlalchemy import orm
from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.ext.declarative import declarative_base
import sqlalchemy as sa
from sqlalchemy_utils import database_exists, create_database


class DbSqlA():
    base        = declarative_base()
    def __init__(self,ConnectionString):
        self.engine             = sa.create_engine(ConnectionString, pool_size=40, max_overflow=40)
        self.session            = scoped_session(sessionmaker(autocommit=False,autoflush=False,bind=self.engine))
        self.base.query         = self.session.query_property()
        self.orm_session        = orm.scoped_session(orm.sessionmaker())(bind=self.engine)
        self.base.metadata.bind = self.engine    

    def create_all(self):
        if not database_exists(self.engine.url):
            create_database(self.engine.url)
        self.base.metadata.create_all(self.engine)


class Database:
    _engines = {
            "SQL Server": "mssql+pyodbc://{}:{}@{}/{}?driver={}",
            "Oracle": "mssql+pyodbc://{}:{}@{}/{}?driver={}"
        }

    def __init__(self, config: DBConfig) -> None:
        pwd = urllib.parse.quote_plus(config.PASSWORD)
        self._connection_string = self._engines[config.db_engine]
        self._connection_string = self._connection_string.format(
            config.USER, pwd, config.SERVER, config.DB_NAME, config.DRIVER
        )
        
    LogWrapping()
    def connect(self):
        db = DbSqlA(self._connection_string)
        return db



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lr_etl',
 'lr_etl.Config',
 'lr_etl.Libs',
 'lr_etl.Libs.db',
 'lr_etl.Models',
 'lr_etl.Models.db',
 'lr_etl.Models.entities',
 'lr_etl.Steps',
 'lr_etl.use_cases']

package_data = \
{'': ['*']}

install_requires = \
['alive-progress>=2.4.1,<3.0.0',
 'pandas>=1.5.1,<2.0.0',
 'paramiko>=2.12.0,<3.0.0',
 'requests>=2.28.1,<3.0.0',
 'sqlalchemy-utils>=0.38.3,<0.39.0',
 'sqlalchemy>=1.4.44,<2.0.0']

setup_kwargs = {
    'name': 'lr-etl',
    'version': '0.0.3',
    'description': '',
    'long_description': '',
    'author': 'FernandoEmerson',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from typing import List
from pandas import DataFrame
from os import listdir
import os
import pandas as pd
from lr_etl.Config import *


class ParseData:

    def __init__(self, dataVersion="v2") -> None:
        self.dataVersion = dataVersion
    
    def parse(self) -> List[DataFrame]:
        """Converte os arquivos csv das listas em uma lista de dataframes"""
        listasStorage = listdir(STORAGE_FOLDER)
        listasStorage = [item for item in listasStorage if '.csv' in item and item not in ['tbl_periodicidade.csv', 'tbl_sublista.csv', 'lista_atencao.csv', 'tbl_fontes.csv']]
        listas = [pd.read_csv(os.path.join(STORAGE_FOLDER, item), encoding="utf8") for item in listasStorage]
        for i, df in enumerate(listas):
            if 'Unnamed: 0' in df.columns:
                listas[i] = df.drop(columns=df.columns[0])
        return listas


from datetime import datetime


class ExecuteRebuild:

    def handle(weekday: int, tablename, conn):
        dt = datetime.now().weekday()
        if dt == weekday:
            conn.session.execute("SET NOCOUNT ON; EXEC PR_INDEX :p1", {"p1": tablename})
            conn.session.commit()